<?php
// includes/vulnerability-checks.php
function shieldcraft_secure_check_vulnerability($name, $version) {
    $search_url = 'https://packetstormsecurity.com/search/?q=wordpress+' . urlencode($name) . '+' . urlencode($version) . '&s=files';
    
    error_log('Vulnerability check search URL: ' . $search_url);
    
    $response = wp_remote_get($search_url);
    
    if (is_wp_error($response)) {
        error_log('Error in vulnerability check response: ' . $response->get_error_message());
        return false;
    }
    
    $html = wp_remote_retrieve_body($response);
    
    error_log('Response HTML: No need for it now ');
    
    $dom = new DOMDocument();
    libxml_use_internal_errors(true);
    $dom->loadHTML($html);
    libxml_clear_errors();
    
    $xpath = new DOMXPath($dom);
    $file_entries = $xpath->query("//dl[contains(@class, 'file')]");
    
    if ($file_entries->length > 0) {
        foreach ($file_entries as $file_entry) {
            $title = trim($xpath->query(".//dt/a", $file_entry)->item(0)->nodeValue);
            $description = trim($xpath->query(".//dd[@class='detail']/p", $file_entry)->item(0)->nodeValue);
            
            if (stripos($title, $name) !== false && stripos($title, $version) !== false) {
                $date = trim($xpath->query(".//dd[@class='datetime']/a", $file_entry)->item(0)->nodeValue);
                $author = trim($xpath->query(".//dd[@class='refer']/a[@class='person']", $file_entry)->item(0)->nodeValue);
                $platform = 'N/A';
                $risk = 'N/A';
                $cve = trim($xpath->query(".//dd[@class='cve']/a", $file_entry)->item(0)->nodeValue);
                
                error_log('Vulnerability found for ' . $name . ' ' . $version . ':');
                error_log('Title: ' . $title);
                error_log('Description: ' . $description);
                error_log('Date: ' . $date);
                error_log('Author: ' . $author);
                error_log('CVE: ' . $cve);
                
                return array(
                    'name' => $name,
                    'version' => $version,
                    'description' => $description,
                    'date' => $date,
                    'platform' => $platform,
                    'risk' => $risk,
                    'cve' => $cve,
                );
            } elseif (stripos($description, $name) !== false && stripos($description, $version) !== false) {
                $date = trim($xpath->query(".//dd[@class='datetime']/a", $file_entry)->item(0)->nodeValue);
                $author = trim($xpath->query(".//dd[@class='refer']/a[@class='person']", $file_entry)->item(0)->nodeValue);
                $platform = 'N/A';
                $risk = 'N/A';
                $cve = trim($xpath->query(".//dd[@class='cve']/a", $file_entry)->item(0)->nodeValue);
                
                error_log('Vulnerability found for ' . $name . ' ' . $version . ':');
                error_log('Title: ' . $title);
                error_log('Description: ' . $description);
                error_log('Date: ' . $date);
                error_log('Author: ' . $author);
                error_log('CVE: ' . $cve);
                
                return array(
                    'name' => $name,
                    'version' => $version,
                    'description' => $description,
                    'date' => $date,
                    'platform' => $platform,
                    'risk' => $risk,
                    'cve' => $cve,
                );
            }
        }
    }
    
    error_log('No vulnerability found for ' . $name . ' ' . $version);
    
    return false;
}
// Perform vulnerability checks
function shieldcraft_secure_check_vulnerabilities() {
    error_log('ShieldCraft Secure: Vulnerability check started.');
    if (!function_exists('get_plugins')) {
        require_once ABSPATH . 'wp-admin/includes/plugin.php';
    }
    $plugins = array_slice(get_plugins(), 0, 5); // Limit to 5 plugins
    $themes = array_slice(wp_get_themes(), 0, 3); // Limit to 3 themes
    update_option('shieldcraft_secure_last_check', current_time('timestamp'));
    $vulnerabilities = array();

    foreach ($plugins as $plugin_file => $plugin_data) {
        $plugin_name = $plugin_data['Name'];
        $plugin_version = $plugin_data['Version'];
        
        $vulnerability = shieldcraft_secure_check_vulnerability($plugin_name, $plugin_version);
        
        if ($vulnerability) {
            $vulnerability['type'] = 'plugin';
            $vulnerabilities[] = $vulnerability;
        }
    }

    foreach ($themes as $theme_slug => $theme) {
        $theme_name = $theme->get('Name');
        $theme_version = $theme->get('Version');
        
        $vulnerability = shieldcraft_secure_check_vulnerability($theme_name, $theme_version);
        
        if ($vulnerability) {
            $vulnerability['type'] = 'theme';
            $vulnerabilities[] = $vulnerability;
        }
    }

    update_option('shieldcraft_secure_vulnerabilities', $vulnerabilities);
    update_option('shieldcraft_secure_last_check', current_time('timestamp'));
    shieldcraft_secure_display_vulnerability_report($vulnerabilities);
    error_log('ShieldCraft Secure: Vulnerability check completed.');
    
    // Send email notification if vulnerabilities are found
    if (!empty($vulnerabilities)) {
        shieldcraft_secure_notify_vulnerabilities($vulnerabilities);
    }
    // Update blocked IPs and user agents lists
    shieldcraft_secure_update_lists();
}

add_action('shieldcraft_secure_check_vulnerabilities', 'shieldcraft_secure_check_vulnerabilities');

add_action('admin_init', 'shieldcraft_secure_handle_manual_check');


function shieldcraft_secure_render_vulnerability_checks_page() {
    $vulnerabilities = get_option('shieldcraft_secure_vulnerabilities', array());
    $last_check_timestamp = get_option('shieldcraft_secure_last_check', 0);
    $last_check_date = $last_check_timestamp ? date_i18n(get_option('date_format'), $last_check_timestamp) : __('Never', 'shieldcraft-secure');
    $last_check_time = $last_check_timestamp ? date_i18n(get_option('time_format'), $last_check_timestamp) : '';
    
    echo '<div class="wrap shieldcraft-secure-wrap">';
    echo '<div class="shieldcraft-secure-title"><h1>' . __('Vulnerability Checks', 'shieldcraft-secure') . '</h1></div>';
    
    // Add the upgrade message
    echo '<div class="shieldcraft-secure-upgrade-message">';
    echo '<p>' . __('To scan more than 5 Plugins and 3 Themes, please purchase the full version.', 'shieldcraft-secure') . '</p>';
    echo '</div>';
    
    // Add the manual check button
    echo '<form method="post" action="">';
    echo '<input type="hidden" name="shieldcraft_secure_manual_check" value="1">';
    echo '<p><input type="submit" class="button button-primary" value="' . __('Run Manual Check', 'shieldcraft-secure') . '">';
    echo ' <span class="last-check-info">' . sprintf(__('Last scheduled check: %s at %s', 'shieldcraft-secure'), $last_check_date, $last_check_time) . '</span>';
    echo '</p>';
    echo '</form>';
    
    echo '<div class="shieldcraft-secure-title"><h2>' . __('Plugins', 'shieldcraft-secure') . '</h2></div>';
    echo '<table class="widefat">';
    echo '<thead>';
    echo '<tr>';
    echo '<th>' . __('Plugin', 'shieldcraft-secure') . '</th>';
    echo '<th>' . __('Vulnerability', 'shieldcraft-secure') . '</th>';
    echo '<th>' . __('Status', 'shieldcraft-secure') . '</th>';
    echo '<th>' . __('CVE', 'shieldcraft-secure') . '</th>';
    echo '<th>' . __('Date', 'shieldcraft-secure') . '</th>';
    echo '</tr>';
    echo '</thead>';
    echo '<tbody>';
    
    $plugins = array_slice(get_plugins(), 0, 5); // Limit to 5 plugins
    foreach ($plugins as $plugin_file => $plugin_data) {
        $plugin_name = $plugin_data['Name'];
        $plugin_version = $plugin_data['Version'];
        $is_vulnerable = false;
        $vulnerability_description = '';
        $cve = '';
        $date = '';
        
        foreach ($vulnerabilities as $vulnerability) {
            if ($vulnerability['name'] === $plugin_name && $vulnerability['version'] === $plugin_version) {
                $is_vulnerable = true;
                $vulnerability_description = $vulnerability['description'];
                $cve = $vulnerability['cve'];
                $date = $vulnerability['date'];
                break;
            }
        }
        
        echo '<tr>';
        echo '<td>' . esc_html($plugin_name . ' ' . $plugin_version) . '</td>';
        echo '<td>' . esc_html($vulnerability_description) . '</td>';
        echo '<td><span style="font-weight: bold; color: ' . ($is_vulnerable ? 'red' : 'green') . ';">' . ($is_vulnerable ? __('VULN', 'shieldcraft-secure') : __('VALID', 'shieldcraft-secure')) . '</span></td>';
        echo '<td>' . ($cve ? '<a href="https://www.google.com/search?q=' . urlencode($cve) . '" target="_blank">' . esc_html($cve) . '</a>' : '') . '</td>';
        echo '<td>' . esc_html($date) . '</td>';
        echo '</tr>';
    }
    
    echo '</tbody>';
    echo '</table>';
    
    echo '<h2>' . __('Themes', 'shieldcraft-secure') . '</h2>';
    echo '<table class="widefat">';
    echo '<thead>';
    echo '<tr>';
    echo '<th>' . __('Theme', 'shieldcraft-secure') . '</th>';
    echo '<th>' . __('Vulnerability', 'shieldcraft-secure') . '</th>';
    echo '<th>' . __('Status', 'shieldcraft-secure') . '</th>';
    echo '<th>' . __('CVE', 'shieldcraft-secure') . '</th>';
    echo '<th>' . __('Date', 'shieldcraft-secure') . '</th>';
    echo '</tr>';
    echo '</thead>';
    echo '<tbody>';
    
    $themes = array_slice(wp_get_themes(), 0, 3); // Limit to 3 themes
    foreach ($themes as $theme) {
        $theme_name = $theme->get('Name');
        $theme_version = $theme->get('Version');
        $is_vulnerable = false;
        $vulnerability_description = '';
        $cve = '';
        $date = '';
        
        foreach ($vulnerabilities as $vulnerability) {
            if ($vulnerability['name'] === $theme_name && $vulnerability['version'] === $theme_version) {
                $is_vulnerable = true;
                $vulnerability_description = $vulnerability['description'];
                $cve = $vulnerability['cve'];
                $date = $vulnerability['date'];
                break;
            }
        }
        
        echo '<tr>';
        echo '<td>' . esc_html($theme_name . ' ' . $theme_version) . '</td>';
        echo '<td>' . esc_html($vulnerability_description) . '</td>';
        echo '<td><span style="font-weight: bold; color: ' . ($is_vulnerable ? 'red' : 'green') . ';">' . ($is_vulnerable ? __('VULN', 'shieldcraft-secure') : __('VALID', 'shieldcraft-secure')) . '</span></td>';
        echo '<td>' . ($cve ? '<a href="https://www.google.com/search?q=' . urlencode($cve) . '" target="_blank">' . esc_html($cve) . '</a>' : '') . '</td>';
        echo '<td>' . esc_html($date) . '</td>';
        echo '</tr>';
    }
    
    echo '</tbody>';
    echo '</table>';
    
    echo '</div>';
}

function shieldcraft_secure_display_vulnerability_report($vulnerabilities) {
    // Display the vulnerability count in the dashboard
    add_action('admin_notices', function() use ($vulnerabilities) {
        $count = count($vulnerabilities);
        if ($count > 0) {
            $message = sprintf(
                _n(
                    'ShieldCraft Secure: %d vulnerability found.',
                    'ShieldCraft Secure: %d vulnerabilities found.',
                    $count,
                    'shieldcraft-secure'
                ),
                $count
            );
            echo "<div class='notice notice-warning'><p>$message</p></div>";
        }
    });
    
    // Display the detailed vulnerability report in the "Vulnerability Checks" page
    add_action('admin_menu', function() {
        add_submenu_page(
            'shieldcraft-secure',
            __('Vulnerability Checks', 'shieldcraft-secure'),
            __('Vulnerability Checks', 'shieldcraft-secure'),
            'manage_options',
            'shieldcraft-secure-vulnerability-checks',
            'shieldcraft_secure_render_vulnerability_checks_page'
        );
    });
}


add_action('admin_init', 'shieldcraft_secure_handle_manual_check');


function shieldcraft_secure_handle_manual_check() {
if (isset($_POST['shieldcraft_secure_manual_check'])) {
shieldcraft_secure_check_vulnerabilities();
wp_redirect(admin_url('admin.php?page=shieldcraft-secure-vulnerability-checks'));
exit;
}
}

// Notify the site admin about the vulnerabilities
function shieldcraft_secure_notify_vulnerabilities($vulnerabilities) {
    update_option('shieldcraft_secure_vulnerabilities', $vulnerabilities);
    
    $to = get_option('shieldcraft_secure_notification_email', get_option('admin_email'));
    
    $subject = __('ShieldCraft Secure - Vulnerability Alert', 'shieldcraft-secure');
    $message = __('The following vulnerabilities were found in your WordPress plugins and themes:', 'shieldcraft-secure') . "\n\n";

    foreach ($vulnerabilities as $vulnerability) {
        $message .= sprintf(
            __('- %s: %s (Version: %s)', 'shieldcraft-secure'),
            ucfirst($vulnerability['type']),
            $vulnerability['name'],
            $vulnerability['version']
        ) . "\n";
        $message .= sprintf(__('  Vulnerability: %s', 'shieldcraft-secure'), $vulnerability['description']) . "\n";
        $message .= sprintf(__('  CVE: %s', 'shieldcraft-secure'), $vulnerability['cve']) . "\n";
        $message .= sprintf(__('  Date: %s', 'shieldcraft-secure'), $vulnerability['date']) . "\n\n";
    }

    $headers = array('Content-Type: text/plain; charset=UTF-8');

    wp_mail($to, $subject, $message, $headers);
}